package CPAN::Indexes;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

$VERSION = '0.02';

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(read_packages read_authors);


=head1 NAME

CPAN::Indexes - Reading the CPAN index files

=head1 SYNOPSIS

  use CPAN::Indexes;

  my($modules) = read_packages($packages_file);
  my($authors) = read_authors($authors_file);


=head1 DESCRIPTION

Very basic functions for reading the CPAN index files.


=head2 Functions

=over 4

=item B<read_packages>

  my($modules) = read_packages($package_file)

Given a CPAN package details file (ie. 02package.details.txt) it will
parse it out and return a hash of package names and their info.

      $modules->{$package_name} = { version     => $version,
                                    file        => $filename,
                                    author      => $author,
                                    comment     => $comment,
                                  };

=cut

sub read_packages {
    my($package_file) = @_;
    my %info = ();
    my %packages = ();

    open(PACK, $package_file) or die "Can't open $package_file:  $!";

    # Read the headers.
    # Mail-header  KEY: VALUE style with blank line ending.
    while(<PACK>) {
        last if /^\s*$/;

        my($key, $val) = m/^(.+?):\s*(.*?)$/;
        $info{$key} = $val;
    }


    warn "Couldn't find a line count.\n" 
      unless defined $info{'Line-Count'};


    my $lines = 0;
    while(<PACK>) {
        chomp;
        $lines++;
        my($mod, $version, $file, $comment) = split " ", $_, 4;

        my($author) = $file =~ m{^.+?/.+?/(.+?)/};

        $packages{$mod} = { version => $version,
                            file    => $file,
                            comment => $comment,
                            author  => $author,
                          };
    }

    warn "Wrong line count!\n" unless $lines == $info{'Line-Count'};

    return \%packages;
}


=item B<read_authors>

  my $authors = read_authors($authors_file);

Given a CPAN authors file (01mailrc.txt) it will return a hash of
author ids and their info.

  $authors->{$id} = {
                     name   => $name,
                     email  => $email,
                    };

=cut

sub read_authors {
    my($authors_file) = shift;

    my %authors = ();

    open(AUTHORS, $authors_file) or die "Can't open $authors_file: $!";
    while( <AUTHORS> ) {
        my($id, $name, $email) = m/^alias \s+ 
                                    (\S+) \s+ 
                                    "([^\"\<]+) \s+ <(.+)>"
                                  /x;

        $authors{$id} = { name => $name, email => $email };
    }

    return \%authors;
}

=back

=head1 AUTHOR

Michael G Schwern E<lt>schwern@pobox.comE<gt>

=head1 SEE ALSO

L<CPAN>

=cut

1;
